/* $Id: misc.c,v 1.7.2.2 2004/08/05 20:21:05 cegger Exp $
******************************************************************************

   LibGG - Misc utility functions

   Copyright (C) 1998 Marcus Sundberg	[marcus@ggi-project.org]

   Permission is hereby granted, free of charge, to any person obtaining a
   copy of this software and associated documentation files (the "Software"),
   to deal in the Software without restriction, including without limitation
   the rights to use, copy, modify, merge, publish, distribute, sublicense,
   and/or sell copies of the Software, and to permit persons to whom the
   Software is furnished to do so, subject to the following conditions:

   The above copyright notice and this permission notice shall be included in
   all copies or substantial portions of the Software.

   THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
   IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
   FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.  IN NO EVENT SHALL
   THE AUTHOR(S) BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER
   IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
   CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.

******************************************************************************
*/

#include <stdlib.h>
#include <string.h>
#include <limits.h>

#include <ggi/gg.h>
#include "plat.h"

#ifndef PATH_MAX
# ifdef _POSIX_PATH_MAX
#  define PATH_MAX _POSIX_PATH_MAX
# else
#  define PATH_MAX 4096 /* Should be enough for most systems */
# endif
#endif

/* FIXME - make win32 and other systems do the right thing here */

#define APPENDNAME	"/.ggi"
#define APPENDLEN	5

const char *ggGetUserDir(void)
{
	static char curpath[PATH_MAX+1];
	char *ptr;
	size_t len;
	int need_ptr2free = 0;

	ggLock(_gg_global_mutex);

	if (curpath[0] != '\0') {
		/* The user dir has already been calculated */
		ggUnlock(_gg_global_mutex);
		return curpath;
	}

	ptr = getenv("HOME");
	if (ptr == NULL) {
		/* hmm... $HOME is not there... */


#ifndef HAVE_TMP_PATH

#if defined(macintosh) || (defined(__MACH__) && defined(__APPLE__))
#define HAVE_TMP_PATH
		/* Darwin 6.x */
		const char *uid = getenv("UID");
		asprintf(&ptr, "/tmp/%s/Temporary Items", uid);
		need_ptr2free = 1;

#endif	/* MAC OS / DARWIN */

#endif


#ifndef HAVE_TMP_PATH

#if defined(_POSIX_SOURCE)
#define HAVE_TMP_PATH
		/* this hopefully catches all unix environments */
		ptr = "/tmp";

#endif	/* Unix */

#endif


#ifndef HAVE_TMP_PATH

#if defined(__WIN32__) && !defined(__CYGWIN__)
#define HAVE_TMP_PATH
		/* Should catch all win32 platforms */
#error Set ptr to an appropriate common default path for win32 platforms.
#endif	/* __WIN32__ */

#endif


#ifndef HAVE_TMP_PATH
		/* Catch all other platforms */
#error You need to set the default temporary path for this system
#endif
	}	/* if */

	len = strlen(ptr);
	if (len + APPENDLEN > PATH_MAX) return NULL;

	strcpy(curpath, ptr);
	strcpy(curpath+len, APPENDNAME);
	ggUnlock(_gg_global_mutex);

	if (need_ptr2free) {
		free(ptr);
	}

	return curpath;
}

#ifndef GG_USLEEP_DEFINED
#error You need to implement ggUSleep() for this system
#endif

#ifndef GG_CURTIME_DEFINED
#error You need to implement ggCurTime() for this system
#endif
